
/*
 * qfits example: Read a value in a table 
 *
 * In this example, a FITS table is loaded and examined
 * using the qfits_table object. Than a value is read and displayed
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "qfits.h"

int main(int argc, char * argv[])
{
    int                     table_type ;
    qfits_table     *       th ;
    qfits_col       *       ch ;
    void            *       column ;
    unsigned char   *       uccol ;
    char            *       ccol ;
    int             *       icol ;
    short           *       scol ;
    float           *       fcol ;
    double          *       dcol ;
    char                    ctmp[512];

    int                     raw ;
    int                     col ;
    int                     extension ;

    int                     i ;

    /* Initialize */
    extension = 1 ;
    raw = 0 ;
    col = 0 ;
    
    /* Check command-line arguments. */
	if (argc<2) {
		printf("use: %s <filename>\n", argv[0]);
		return -1 ;
	}

    /* Check if this is indeed a FITS file */
    if (is_fits_file(argv[1])!=1) {
        printf("%s is not a FITS file\n", argv[1]);
        return -1 ;
    }

    /* Check if the FITS file is a FITS table */
    table_type = qfits_is_table(argv[1], extension) ;
    if (table_type == QFITS_INVALIDTABLE) {
        printf("%s is not a TFITS table\n", argv[1]) ;
        return -1 ;
    } else if (table_type == QFITS_ASCIITABLE) {
        printf("%s is an ASCII table\n", argv[1]) ;
    } else if (table_type == QFITS_BINTABLE) {
        printf("%s is a BIN table\n", argv[1]) ;
    } else return -1 ;

    /* Get the positions in the table from the command line or the defaults */
    if (argc>3) {
        raw = (int)atoi(argv[2]) - 1 ;
        col = (int)atoi(argv[3]) - 1 ;
    } else printf("Use defaults\n") ;
    printf("Get the value of : raw: %d, col: %d\n", raw+1, col+1) ;

    /* Load the table informations */
    /* The th objects contains all needed infos about the table */
    if ((th = qfits_table_open(argv[1], extension)) == NULL) {
        printf("cannot get infos from the table\n") ;
        return -1 ;
    } 

    /* Test if col and raw are valid */
    if ((raw < 0) || (col < 0) || (col+1 > th->nc) || (raw+1 > th->nr)) { 
        printf("invalid specified raw and col\n") ;
        qfits_table_close(th) ;
        return -1 ;
    }
    
    /* Load the needed column as a void*  */
    if ((column = qfits_query_column_data(th, col, NULL, NULL)) == NULL) {
        printf("Cannot access to the column %d\n", col+1) ;
        qfits_table_close(th) ;
        return -1 ;
    }
   
    /* Set reference to the column */
    ch = th->col + col ;
            
    printf("The requested field is: \n") ;
    /* column is different for each type */
    /* For each type the column has to be casted in the right type */
    /* The searched value can than be read */
    switch(ch->atom_type) {
        case TFITS_BIN_TYPE_A:
        case TFITS_ASCII_TYPE_A:
            ccol = (char*)column ;
            strncpy(ctmp, ccol + ch->atom_nb * raw, ch->atom_nb) ;
            ctmp[ch->atom_size*ch->atom_nb] = (char)0 ;
            printf("%s", ctmp) ;
            break ;
        case TFITS_BIN_TYPE_B:
        case TFITS_BIN_TYPE_X:
            uccol = (unsigned char*)column ;
            for (i=0 ; i<ch->atom_nb ; i++) {
                printf("%d\t", (int)uccol[ch->atom_nb*raw+i]) ;
            }
            break ;
        case TFITS_BIN_TYPE_D:
        case TFITS_BIN_TYPE_M:
            dcol = (double*)column ;
            for (i=0 ; i<ch->atom_nb ; i++) {
                printf("%g\t", dcol[ch->atom_nb*raw+i]) ;
            }
            break ;
        case TFITS_BIN_TYPE_E:
        case TFITS_BIN_TYPE_C:
            fcol = (float*)column ;
            for (i=0 ; i<ch->atom_nb ; i++) {
                printf("%f\t", fcol[ch->atom_nb*raw+i]) ;
            }
            break ;
        case TFITS_BIN_TYPE_I:
            scol = (short*)column ;
            for (i=0 ; i<ch->atom_nb ; i++) {
                 printf("%d\t", (int)scol[ch->atom_nb*raw+i]) ;
            }
            break ;
        case TFITS_BIN_TYPE_J:
        case TFITS_BIN_TYPE_P:
            icol = (int*)column ;
            for (i=0 ; i<ch->atom_nb ; i++) {
                printf("%d\t", icol[ch->atom_nb*raw+i]) ;
            }
            break ;
        case TFITS_BIN_TYPE_L:
            ccol = (char*)column ;
            for (i=0 ; i<ch->atom_nb ; i++) {
                printf("%c\t", ccol[ch->atom_nb*raw+i]) ;
            }
            break ;
        case TFITS_ASCII_TYPE_I:
            icol = (int*)column ;
            printf("%d", icol[raw]) ;
            break ;
        case TFITS_ASCII_TYPE_E:
        case TFITS_ASCII_TYPE_F:
            fcol = (float*)column ;
            printf("%f", fcol[raw]) ;
            break ;
        case TFITS_ASCII_TYPE_D:
            dcol = (double*)column ;
            printf("%g", dcol[raw]) ;
            break ;
        default:
            printf("Unrecognized type\n") ;
            free(column) ;
            qfits_table_close(th) ;
            return -1 ;
            break ;
    }
    printf("\n") ;
    free(column) ;
    return 0 ;
}

